@@ -19,7 +19,7 @@ use Encode 'encode', 'decode';
 
 use Carp 'confess';
 
-our $VERSION = '0.0445';
+our $VERSION = '0.0702';
 
 my $SPVM_ENV;
 my $BUILDER;
@@ -31,25 +31,32 @@ sub import {
   my ($class, $package_name) = @_;
   
   unless ($BUILDER) {
-    my $build_dir;
-    if (defined $ENV{SPVM_BUILD_DIR}) {
-      $build_dir = $ENV{SPVM_BUILD_DIR};
-    }
-    else {
-      $build_dir = "$FindBin::Bin/spvm_build";
-    }
+    my $build_dir = $ENV{SPVM_BUILD_DIR};
     $BUILDER = SPVM::Builder->new(build_dir => $build_dir);
   }
 
   # Add package informations
   if (defined $package_name) {
-    my ($file, $line) = (caller)[1, 2];
-    my $package_info = {
-      name => $package_name,
-      file => $file,
-      line => $line
-    };
-    push @{$BUILDER->{package_infos}}, $package_info;
+    if ($package_name =~ /\.spvm$/) {
+      my $file = $package_name;
+      my $line = (caller)[2];
+      
+      my $package_info = {
+        name => 'Main',
+        file => $file,
+        line => $line
+      };
+      push @{$BUILDER->{package_infos}}, $package_info;
+    }
+    else {
+      my ($file, $line) = (caller)[1, 2];
+      my $package_info = {
+        name => $package_name,
+        file => $file,
+        line => $line
+      };
+      push @{$BUILDER->{package_infos}}, $package_info;
+    }
   }
 }
 
@@ -107,7 +114,7 @@ sub new_mulnum_array_from_bin { SPVM::ExchangeAPI::new_mulnum_array_from_bin($SP
 
 =head1 NAME
 
-SPVM - Static Perl Virtual Machine. Fast calculation & Easy C/C++ Binding
+SPVM - Static Perl Virtual Machine. Fast calculation, fast array operation, and easy C/C++ Binding.
 
 =head1 SYNOPSIS
 
@@ -192,9 +199,9 @@ L<http://jp.spvm.info/exchange-api.html>
 
 L<SPVM::Document::NativeAPI> - Native API is C level functions to manipulate SPVM data.
 
-=head2 Extension
+=head2 NativeAPI
 
-L<SPVM::Document::Extension> - Extension is the way to C/C++ Binding to SPVM
+L<SPVM::Document::NativeAPI> - NativeAPI is the way to C/C++ Binding to SPVM
 
 =head2 Language Specification
 
@@ -350,7 +357,7 @@ Above is same as the following.
   my $obj : Foo = new Foo;
   my $values : int[3] = new int[3];
 
-=head2 C Extension using SPVM
+=head2 C NativeAPI using SPVM
 
 SPVM Module:
 
@@ -393,7 +400,7 @@ C Source File;
     return SPVM_SUCCESS;
   }
 
-Use Extension Module from Perl:
+Use NativeAPI Module from Perl:
 
   use FindBin;
   use lib "$FindBin::Bin/lib";
@@ -409,7 +416,7 @@ Use Extension Module from Perl:
   
   print $total . "\n";
 
-See also L<SPVM::Document::Extension>, L<SPVM::Document::NativeAPI>.
+See also L<SPVM::Document::NativeAPI>, L<SPVM::Document::NativeAPI>.
 
 =head1 ENVIRONMENT VARIABLE
 
@@ -417,9 +424,15 @@ See also L<SPVM::Document::Extension>, L<SPVM::Document::NativeAPI>.
 
 SPVM build directory for precompile and native subroutine.
 
-If SPVM_BUILD_DIR environment variable is not specified, spvm_build directory of script directory is set to build directory.
+If SPVM_BUILD_DIR environment variable is not set, SPVM can't compile precompile subroutine and native subroutine, and a exception occur. You see error message "SPVM_BUILD_DIR environment variable must be set ...".
 
-For exmple, If your script is placed at "/path/app.pl", build directory is "/path/spvm_build".
+In bash, you can set SPVM_BUILD_DIR to the following.
+
+  export SPVM_BUILD_DIR=~/.spvm_build
+
+In bash, you can set SPVM_BUILD_DIR to the following.
+
+  export SPVM_BUILD_DIR=~/.spvm_build
 
 =head1 NOTE
 
