@@ -4,6 +4,26 @@ use strict;
 use warnings;
 use Config;
 
+sub get_include_dirs {
+  my ($self, $include_dirs) = @_;
+  
+  return $self->{include_dirs};
+}
+
+sub set_include_dirs {
+  my ($self, $include_dirs) = @_;
+  
+  $self->{include_dirs} = $include_dirs;
+  
+  return $self;
+}
+
+sub add_include_dir {
+  my ($self, $include_dir) = @_;
+  
+  push @{$self->{include_dirs}}, $include_dir;
+}
+
 sub get_ext {
   my ($self, $ext) = @_;
   
@@ -18,12 +38,78 @@ sub set_ext {
   return $self;
 }
 
+sub get_quiet {
+  my ($self, $quiet) = @_;
+  
+  return $self->{quiet};
+}
+
+sub set_quiet {
+  my ($self, $quiet) = @_;
+  
+  $self->{quiet} = $quiet;
+  
+  return $self;
+}
+
+sub set_extra_compiler_flags {
+  my ($self, $extra_compiler_flags) = @_;
+  
+  $self->{extra_compiler_flags} = $extra_compiler_flags;
+  
+  return $self;
+}
+
+sub get_extra_compiler_flags {
+  my $self = shift;
+  
+  return $self->{extra_compiler_flags};
+}
+
+sub add_extra_compiler_flags {
+  my ($self, $new_extra_compiler_flags) = @_;
+  
+  my $extra_compiler_flags = $self->{extra_compiler_flags};
+  $extra_compiler_flags = '' unless defined $extra_compiler_flags;
+  
+  $extra_compiler_flags .= " $new_extra_compiler_flags";
+  
+  $self->{extra_compiler_flags} = $extra_compiler_flags;
+}
+
+sub set_extra_linker_flags {
+  my ($self, $extra_linker_flags) = @_;
+  
+  $self->{extra_linker_flags} = $extra_linker_flags;
+  
+  return $self;
+}
+
+sub get_extra_linker_flags {
+  my $self = shift;
+  
+  return $self->{extra_linker_flags};
+}
+
+sub add_extra_linker_flags {
+  my ($self, $new_extra_linker_flags) = @_;
+  
+  my $extra_linker_flags = $self->{extra_linker_flags};
+  $extra_linker_flags = '' unless defined $extra_linker_flags;
+  
+  $extra_linker_flags .= " $new_extra_linker_flags";
+  
+  $self->{extra_linker_flags} = $extra_linker_flags;
+}
+
 sub new {
   my $class = shift;
   
   my $self = {};
   
   $self->{config} = {};
+  
+  $self->{include_dirs} = [];
 
   return bless $self, $class;
 }
@@ -66,11 +152,19 @@ sub new_c99 {
   my $default_config = {%Config};
   $bconf->replace_all_config($default_config);
   
+  # Remove and get include dir from ccflags
+  my @ccflags_include_dirs = $bconf->_remove_include_dirs_from_ccflags;
+  
   # Add include directory to ccflags
   my $include_dir = $INC{"SPVM/Builder/Config.pm"};
   $include_dir =~ s/\/Config\.pm$//;
   $include_dir .= '/include';
-  $bconf->add_extra_compiler_flags("-I$include_dir");
+  $bconf->add_include_dir($include_dir);
+  
+  # Add ccflags include dir
+  for my $ccflags_include_dir (@ccflags_include_dirs) {
+    $bconf->add_include_dir($ccflags_include_dir);
+  }
   
   # C99
   $bconf->set_std('c99');
@@ -78,7 +172,7 @@ sub new_c99 {
   # Optimize
   $bconf->set_optimize('-O3');
   
-  # Extension
+  # NativeAPI
   $bconf->set_ext('c');
   
   # I want to print warnings, but if gcc version is different, can't suppress no needed warning message.
@@ -98,12 +192,20 @@ sub new_cpp {
   # Use default config
   my $default_config = {%Config};
   $bconf->replace_all_config($default_config);
+
+  # Remove and get include dir from ccflags
+  my @ccflags_include_dirs = $bconf->_remove_include_dirs_from_ccflags;
   
   # Add include directory to ccflags
   my $include_dir = $INC{"SPVM/Builder/Config.pm"};
   $include_dir =~ s/\/Config\.pm$//;
   $include_dir .= '/include';
-  $bconf->add_extra_compiler_flags("-I$include_dir");
+  $bconf->add_include_dir($include_dir);
+  
+  # Add ccflags include dir
+  for my $ccflags_include_dir (@ccflags_include_dirs) {
+    $bconf->add_include_dir($ccflags_include_dir);
+  }
   
   # Optimize
   $bconf->set_optimize('-O3');
@@ -114,25 +216,22 @@ sub new_cpp {
   # LD
   $bconf->set_ld('g++');
   
-  # Extension
+  # NativeAPI
   $bconf->set_ext('cpp');
   
-  # Delete std
-  $bconf->delete_std;
-  
   return $bconf;
 }
 
-sub get_cache {
-  my ($self, $cache) = @_;
+sub get_no_cache {
+  my ($self, $use_cache) = @_;
   
-  return $self->{cache};
+  return $self->{use_cache};
 }
 
-sub set_cache {
-  my ($self, $cache) = @_;
+sub set_no_cache {
+  my ($self, $use_cache) = @_;
   
-  $self->{cache} = $cache;
+  $self->{use_cache} = $use_cache;
   
   return $self;
 }
@@ -194,15 +293,16 @@ sub add_ccflags {
 sub set_std {
   my ($self, $spec) = @_;
   
-  my $ccflags = $self->get_ccflags;
+  my $extra_compiler_flags = $self->get_extra_compiler_flags;
+  $extra_compiler_flags = '' unless defined $extra_compiler_flags;
   
   # Remove -std=foo section
-  $ccflags =~ s/-std=[^ ]+//g;
+  $extra_compiler_flags =~ s/-std=[^ ]+//g;
   
-  $ccflags .= " -std=$spec";
+  $extra_compiler_flags .= " -std=$spec";
   
   # Add -std=foo section
-  $self->set_ccflags($ccflags);
+  $self->set_extra_compiler_flags($extra_compiler_flags);
   
   return $self;
 }
@@ -210,13 +310,13 @@ sub set_std {
 sub delete_std {
   my ($self) = @_;
   
-  my $ccflags = $self->get_ccflags;
+  my $extra_compiler_flags = $self->get_extra_compiler_flags;
   
   # Remove -std=foo section
-  $ccflags =~ s/-std=[^ ]+//g;
+  $extra_compiler_flags =~ s/-std=[^ ]+//g;
   
   # Add -std=foo section
-  $self->set_ccflags($ccflags);
+  $self->set_extra_compiler_flags($extra_compiler_flags);
   
   return $self;
 }
@@ -283,71 +383,32 @@ sub add_lddlflags {
   return $self;
 }
 
-sub set_extra_compiler_flags {
-  my ($self, $extra_compiler_flags) = @_;
+sub _remove_include_dirs_from_ccflags {
+  my ($self) = @_;
   
-  $self->set_config(extra_compiler_flags => $extra_compiler_flags);
+  my $ccflags = $self->get_ccflags;
   
-  return $self;
+  my @parts = split(/ +/, $ccflags);
+  
+  my @rest_parts;
+  my @include_dirs;
+  for my $part (@parts) {
+    if ($part =~ /^-I(.*)/) {
+      my $include_dir = $1;
+      push @include_dirs, $include_dir;
+    }
+    else {
+      push @rest_parts, $part;
+    }
+  }
+  
+  my $rest_ccflags = join(' ', @rest_parts);
+  
+  $self->set_ccflags($rest_ccflags);
+  
+  return @include_dirs;
 }
 
-sub get_quiet {
-  my ($self, $quiet) = @_;
-  
-  return $self->{quiet};
-}
-
-sub set_quiet {
-  my ($self, $quiet) = @_;
-  
-  $self->{quiet} = $quiet;
-  
-  return $self;
-}
-
-sub get_extra_compiler_flags {
-  my $self = shift;
-  
-  return $self->get_config('extra_compiler_flags');
-}
-
-sub add_extra_compiler_flags {
-  my ($self, $new_extra_compiler_flags) = @_;
-  
-  my $extra_compiler_flags = $self->get_config('extra_compiler_flags');
-  
-  $extra_compiler_flags .= " $new_extra_compiler_flags";
-  
-  $self->set_config('extra_compiler_flags' => $extra_compiler_flags);
-  
-  return $self;
-}
-
-sub set_extra_linker_flags {
-  my ($self, $extra_linker_flags) = @_;
-  
-  $self->set_config(extra_linker_flags => $extra_linker_flags);
-  
-  return $self;
-}
-
-sub get_extra_linker_flags {
-  my $self = shift;
-  
-  return $self->get_config('extra_linker_flags');
-}
-
-sub add_extra_linker_flags {
-  my ($self, $new_extra_linker_flags) = @_;
-  
-  my $extra_linker_flags = $self->get_config('extra_linker_flags');
-  
-  $extra_linker_flags .= " $new_extra_linker_flags";
-  
-  $self->set_config('extra_linker_flags' => $extra_linker_flags);
-  
-  return $self;
-}
 
 1;
 
@@ -394,31 +455,21 @@ Get a config value.
 
 Set a config value.
 
-=head2 set_ccflags
-
-  $bconf->set_ccflags($ccflags);
-
-Set C<ccflags>.
-
-=head2 get_ccflags
-
-  my $ccflags = $bconf->get_ccflags;
-
-Get C<ccflags>.
-
-=head2 add_ccflags
-
-  $bconf->add_ccflags($ccflags);
-
-Add C<ccflags> after current C<ccflags>.
-
 =head2 set_std
 
   $bconf->set_std('gnu99');
 
 Set C<std>.
 
-Internally, remove C<-std=old> and add C<-std=new> after C<ccflags>.
+Internally, remove C<-std=old> if exists and add C<-std=new> after C<extra_compiler_flags>.
+
+=head2 delete_std
+
+  $bconf->delete_std;
+
+Delete C<std>.
+
+Internally, remove C<-std=old> if exists from C<extra_compiler_flags>.
 
 =head2 set_cc
 
@@ -479,3 +530,53 @@ Add C<lddlflags> after current C<lddlflags>.
   my $bconf = SPVM::Builder::Config->new_c99;
 
 Create defaulgt build config. This is L<SPVM::Builder::Config> object.
+
+=head2 get_extra_compiler_flags
+
+  my $extra_compiler_flags = $bconf->get_extra_compiler_flags;
+
+Get C<extra_compiler_flags>.
+
+=head2 set_extra_compiler_flags
+
+  $bconf->set_extra_compiler_flags($extra_compiler_flags);
+
+Set C<extra_compiler_flags>.
+
+=head2 add_extra_compiler_flags
+
+  $bconf->add_extra_compiler_flags($extra_compiler_flags);
+
+Add new C<extra_compiler_flags> after current C<extra_compiler_flags>.
+
+=head2 get_extra_linker_flags
+
+  my $extra_linker_flags = $bconf->get_extra_linker_flags;
+
+Get C<extra_linker_flags>.
+
+=head2 set_extra_linker_flags
+
+  $bconf->set_extra_linker_flags($extra_linker_flags);
+
+Set C<extra_linker_flags>.
+
+=head2 add_extra_linker_flags
+
+  $bconf->add_extra_linker_flags($extra_linker_flags);
+
+Add new C<extra_linker_flags> after current C<extra_linker_flags>.
+
+=head2 get_include_dirs
+
+  my $include_dirs = $bconf->get_include_dirs;
+
+Get include directories. This value is array refernce.
+
+=head2 set_include_dirs
+
+  $bconf->set_include_dirs($include_dirs);
+
+=head2 add_include_dir
+
+  $bconf->add_include_dir($include_dir);
